// Proiect PM 2021
// Broloboc - Mihalcea Adrian, 334CA

#include <Wire.h>    // For I2C communication
#include <SevSeg.h>  // 7-segment library

#define BUZZER A0
#define ACCEPTABLE 2

SevSeg sevseg;
// MPU6150 I2C address
int gyro = 0x68;
float GyroX, GyroY, GyroZ;
float gyroAngleX, gyroAngleY, gyroAngleZ;
float errorX, errorY, errorZ;
float elapsedTime, currentTime, previousTime;

void setup() {
	pinMode(A0, OUTPUT);
	Serial.begin(9600);
	Wire.begin();                      // Initialize comunication
	Wire.beginTransmission(gyro);       // Start communication with MPU6050 // MPU=0x68
	Wire.write(0x6B);                  // Talk to the register 6B
	Wire.write(0x00);                  // Make reset - place a 0 into the 6B register
	Wire.endTransmission(true);        //end the transmission

	// When first using the gyroscope, this can detect its biases and account for
	// them
	// print_error();

	// Configures the 7-segment display
	byte numDigits = 4;  
	byte digitPins[] = {2, 3, 4, 5};
	byte segmentPins[] = {6, 7, 8, 9, 10, 11, 12, 13};
	sevseg.begin(COMMON_CATHODE, numDigits, digitPins, segmentPins, 0);
	sevseg.setBrightness(100);
	sevseg.setNumber(0000, 3);
	delay(200);
}

void loop() {
	sevseg.refreshDisplay();

	previousTime = currentTime;
	currentTime = millis();
	elapsedTime = (currentTime - previousTime) / 1000;

	// read 6 registers starting from 0x43 address
	// each axis data is in two registers
	Wire.beginTransmission(gyro);
	Wire.write(0x43); // First register we need has address 0x43
	Wire.endTransmission(false);
	Wire.requestFrom(MPU, 6, true);

	// we divide the value by 131, according to the datasheet
	GyroX = (Wire.read() << 8 | Wire.read()) / 131.0;
	GyroY = (Wire.read() << 8 | Wire.read()) / 131.0;
	GyroZ = (Wire.read() << 8 | Wire.read()) / 131.0;

	// Correcting the value with the calculated error
	GyroX = GyroX - 0.61; // errorX ~(0.61)

	// deg / s * s = deg
	gyroAngleX = gyroAngleX + GyroX * elapsedTime; 
	//  gyroAngleY = gyroAngleY + GyroY * elapsedTime;

	// Display onto 7-segment display
	int display = gyroAngleX * 100;
	if (display < 0)
		display = -display;
	sevseg.setNumber(display, 2);
	sevseg.refreshDisplay();

	// Turn buzzer on, if necessary
	if (display < 100 * ACCEPTABLE)
		tone(BUZZER, 33);
	else
		noTone(BUZZER);

	// Print the values on the serial monitor
	// Serial.println(gyroAngleX);
	// Serial.println(gyroAngleY);
}

// Called optionally in the setup to calibrate gyroscope
void print_error() {
	int c = 0;
	// Read gyro values 200 times
	while (c < 200) {
		Wire.beginTransmission(gyro);
		Wire.write(0x43);
		Wire.endTransmission(false);
		Wire.requestFrom(MPU, 6, true);
		GyroX = Wire.read() << 8 | Wire.read();
		GyroY = Wire.read() << 8 | Wire.read();
		GyroZ = Wire.read() << 8 | Wire.read();
		// Sum all readings
		errorX = errorX + (GyroX / 131.0);
		errorY = errorY + (GyroY / 131.0);
		errorZ = errorZ + (GyroZ / 131.0);
		c++;
	}

	errorX = errorX / 200;
	errorY = errorY / 200;
	errorZ = errorZ / 200;

	Serial.print("errorX: ");
	Serial.println(errorX);
	Serial.print("errorY: ");
	Serial.println(errorY);
	Serial.print("errorZ: ");
	Serial.println(errorZ);
}